/***************************************************************************
 *   Copyright (C) 2004 by Trenton Carr                                    *
 *   trentoncarr@telkomsa.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <math.h> //for log10(), pow(), modf()
#include <kdebug.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qstring.h>

#include "ktrigplotwidget.h"

KTrigPlotWidget::KTrigPlotWidget( double x1, double x2, double y1, double y2, QWidget *parent, const char* name )
        : KPlotWidget( x1, x2, y1, y2, parent, name )
{}

void KTrigPlotWidget::drawBox( QPainter *p )
{
    //First, fill in padding region with bgColor() to mask out-of-bounds plot data
    p->setPen( bgColor() );
    p->setBrush( bgColor() );

    //left padding ( don't forget: we have translated by XPADDING, YPADDING )
    p->drawRect( -leftPadding(), -topPadding(), leftPadding(), height() );

    //right padding
    p->drawRect( PixRect.width(), -topPadding(), rightPadding(), height() );

    //top padding
    p->drawRect( 0, -topPadding(), PixRect.width(), topPadding() );

    //bottom padding
    p->drawRect( 0, PixRect.height(), PixRect.width(), bottomPadding() );

    if ( ShowCartTrigAxes )
    {
        p->save();

        p->setPen( fgColor() );

        p->translate( 210, 210 );		//Change Coords to reflect true
        p->rotate( 270 );			//Cartesian system (0,0)

        p->drawLine( 0,-210 ,0 ,210 );		// Draw Y-Axis
        p->drawLine( -210, 0, 210, 0 );		// Draw X-Axis

        p->save();

        for ( int i = 0; i < 72; i++ )
        {
            p->setPen( gridColor() );
            p->rotate( 5 );
            p->drawLine( 148, 148, 152, 152 );	// Draw 5 Deg Tick Marks
        }
        p->restore();
        p->setBrush( Qt::NoBrush );
        p->drawEllipse ( -210, -210, 420, 420 );// Draw Trig Cirkle
    }

    p->restore();

    if ( ShowGrid )
    {
        //Grid lines are placed at locations of primary axes' major tickmarks
        p->setPen( gridColor() );

        //vertical grid lines
        double x0 = x() - dmod( x(), dXtick ); //zeropoint; x(i) is this plus i*dXtick1
        for ( int ix = 0; ix <= nmajX+1; ix++ )
        {
            int px = int( PixRect.width() * ( (x0 + ix*dXtick - x())/dataWidth() ) );
            p->drawLine( px, 0, px, PixRect.height() );
        }

        //horizontal grid lines
        double y0 = y() - dmod( y(), dYtick ); //zeropoint; y(i) is this plus i*mX
        for ( int iy = 0; iy <= nmajY+1; iy++ )
        {
            int py = int( PixRect.height() * ( (y0 + iy*dYtick - y())/dataHeight() ) );
            p->drawLine( 0, py, PixRect.width(), py );
        }
    }

    p->setPen( fgColor() );
    p->setBrush( Qt::NoBrush );

    if ( ShowAxes )
        p->drawRect( PixRect ); //box outline

    if ( ShowTickMarks )
    {
        //spacing between minor tickmarks (in data units)
        double dminX = dXtick/nminX;
        double dminY = dYtick/nminY;

        //set small font for tick labels
        QFont f = p->font();
        int s = f.pointSize();
        f.setPointSize( s - 2 );
        p->setFont( f );

        //--- Draw X tickmarks---//
        double x0 = x() - dmod( x(), dXtick ); //zeropoint; tickmark i is this plus i*dXtick (in data units)
        if ( x() < 0.0 )
            x0 -= dXtick;

        for ( int ix = 0; ix <= nmajX+1; ix++ )
        {
            //position of tickmark i (in screen units)
            int px = int( PixRect.width() * ( (x0 + ix*dXtick - x() )/dataWidth() ) );

            if ( px > 0 && px < PixRect.width() )
            {
                p->drawLine( px, PixRect.height() - 2, px, PixRect.height() - BIGTICKSIZE - 2 );
                p->drawLine( px, 0, px, BIGTICKSIZE );
            }

            //tick label
            if ( ShowTickLabels )
            {
                double lab = x0 + ix*dXtick;
                if ( fabs(lab)/dXtick < 0.00001 )
                    lab = 0.0; //fix occassional roundoff error with "0.0" label

                QString str = QString( "%1" ).arg( lab, 0, 'g', 2 );
                if ( px > 0 && px < PixRect.width() )
                {
                    QRect r( px - BIGTICKSIZE, PixRect.height()+BIGTICKSIZE, 2*BIGTICKSIZE, BIGTICKSIZE );
                    p->drawText( r, Qt::AlignCenter | Qt::DontClip, str );
                }
            }

            //draw minor ticks
            for ( int j=0; j < nminX; j++ )
            {
                //position of minor tickmark j (in screen units)
                int pmin = int( px + PixRect.width()*j*dminX/dataWidth() );

                if ( pmin > 0 && pmin < PixRect.width() )
                {
                    p->drawLine( pmin, PixRect.height() - 2, pmin, PixRect.height() - SMALLTICKSIZE - 2 );
                    p->drawLine( pmin, 0, pmin, SMALLTICKSIZE );
                }
            }
        }

        //--- Draw Y tickmarks---//
        double y0 = y() - dmod( y(), dYtick ); //zeropoint; tickmark i is this plus i*dYtick1 (in data units)
        if ( y() < 0.0 )
            y0 -= dYtick;

        for ( int iy = 0; iy <= nmajY+1; iy++ )
        {
            //position of tickmark i (in screen units)
            int py = PixRect.height() - int( PixRect.height() * ( (y0 + iy*dYtick - y())/dataHeight() ) );
            if ( py > 0 && py < PixRect.height() )
            {
                p->drawLine( 0, py, BIGTICKSIZE, py );
                p->drawLine( PixRect.width()-2, py, PixRect.width()-BIGTICKSIZE-2, py );
            }

            //tick label
            if ( ShowTickLabels )
            {
                double lab = y0 + iy*dYtick;
                if ( fabs(lab)/dYtick < 0.00001 )
                    lab = 0.0; //fix occassional roundoff error with "0.0" label

                QString str = QString( "%1" ).arg( lab, 0, 'g', 2 );
                if ( py > 0 && py < PixRect.height() )
                {
                    QRect r( -2*BIGTICKSIZE, py-SMALLTICKSIZE, 2*BIGTICKSIZE, 2*SMALLTICKSIZE );
                    p->drawText( r, Qt::AlignCenter | Qt::DontClip, str );
                }
            }

            //minor ticks
            for ( int j=0; j < nminY; j++ )
            {
                //position of minor tickmark j (in screen units)
                int pmin = int( py - PixRect.height()*j*dminY/dataHeight() );
                if ( pmin > 0 && pmin < PixRect.height() )
                {
                    p->drawLine( 0, pmin, SMALLTICKSIZE, pmin );
                    p->drawLine( PixRect.width()-2, pmin, PixRect.width()-SMALLTICKSIZE-2, pmin );
                }
            }
        } //end draw Y tickmarks
    } //end if ( ShowTickMarks )

    //Draw X Axis Label
    if ( ! XAxisLabel.isEmpty() )
    {
        QRect r( 0, PixRect.height() + 2*YPADDING, PixRect.width(), YPADDING );
        p->drawText( r, Qt::AlignCenter, XAxisLabel );
    }

    //Draw Y Axis Label.  We need to draw the text sideways.
    if ( ! YAxisLabel.isEmpty() )
    {
        //store current painter translation/rotation state
        p->save();

        //translate coord sys to left corner of axis label rectangle, then rotate 90 degrees.
        p->translate( -3*XPADDING, PixRect.height() );
        p->rotate( -90.0 );

        QRect r( 0, 0, PixRect.height(), XPADDING );
        p->drawText( r, Qt::AlignCenter, YAxisLabel ); //draw the label, now that we are sideways

        p->restore();  //restore translation/rotation state
    }
}
#include "ktrigplotwidget.moc"
