/***************************************************************************
 *   Copyright (C) 2004 by Trenton Carr                                    *
 *   trentoncarr@telkomsa.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlayout.h>
#include <klocale.h>
#include <knuminput.h>

#include "trigviewer.h"
#include "ktrigplotwidget.h"
#include "deg_theta.h"


TrigViewer::TrigViewer(QWidget* parent, const char* name)
        : QWidget( parent, name )

{
    QVBoxLayout *vlay = new QVBoxLayout( this, 0, -1 );

    KDoubleSpinBox *spin = new KDoubleSpinBox( 0.0, 360.0, 1, 0.0, 2, this, 0);
    
    Deg_Theta *deg = new Deg_Theta();
    
    connect( spin, SIGNAL(valueChanged( double )), deg, SLOT(setAngle( double )));
    
    deg_theta = deg->angle();    

    pw = new KTrigPlotWidget( -100.0, 100.0, -100.0, 100.0, this );

    pw->setShowAxes( true );
    pw->setShowTickMarks( true );
    pw->setBGColor( QColor( "white" ) );
    pw->setFGColor( QColor ( "black") );
    pw->setXAxisLabel( i18n( "X-Axis" ) );
    pw->setYAxisLabel( i18n( "Y-Axis" ) );
    pw->setShowCartTrigAxes( true );
    pw->setShowCartTrigTickMarks( true );		//Might not be needed

    vlay->addWidget( pw );
    vlay->addWidget( spin );
    resize( 500, 500 );

    drawObjects();
}

TrigViewer::~TrigViewer()
{}


void TrigViewer::paintEvent( QPaintEvent* )
{
    pw->update();
}


void TrigViewer::drawObjects()
{


    KPlotObject *r_hypotenuse;				//3 sides of triangle
    KPlotObject *x_adjacent;
    KPlotObject *y_opposite;
    
    double radians = deg_theta * M_PI / 180;		//thank GNU for manpages, three days looking for this

    double X_val = 100*cos(radians);			//Find X Value
    double Y_val = 100*sin(radians);			//Find Y Value

    r_hypotenuse = new KPlotObject( "R", "black", KPlotObject::POLYGON, 2, KPlotObject::SOLID );
    r_hypotenuse->addPoint( new DPoint( 0.0, 0.0 ) );
    r_hypotenuse->addPoint( new DPoint( X_val, Y_val ) );
    pw->addObject( r_hypotenuse );			//Draw R

    x_adjacent = new KPlotObject( "X", "black", KPlotObject::POLYGON, 2, KPlotObject::SOLID );
    x_adjacent->addPoint( new DPoint( 0.0, 0.0 ) );
    x_adjacent->addPoint( new DPoint( X_val, 0.0 ) );
    pw->addObject( x_adjacent );			//Draw X

    y_opposite = new KPlotObject( "Y", "black", KPlotObject::POLYGON, 2, KPlotObject::SOLID );
    y_opposite->addPoint( new DPoint( X_val, 0.0 ) );
    y_opposite->addPoint( new DPoint( X_val, Y_val ) );
    pw->addObject( y_opposite );			//Draw Y
}




#include "trigviewer.moc"

